unit PicButton;

interface
uses
  Windows, Messages, Classes, Graphics, Controls, stdCtrls, Dialogs, SysUtils;


type
TPicButton = class(TButton)
    private
    FOver, FDoRect, FDoText: Boolean;
    FFocusColor: TColor;
    FNormalPic, FHiLightPic, FDownPic, FDisabledPic, FFocusPic: TBitmap;
    procedure setNormal(Value: TBitMap);
    procedure setHiLight(Value: TBitMap);
    procedure setDown(Value: TBitMap);
    procedure setDisabled(Value: TBitmap);
    procedure setFocusPic(Value: TBitMap);
    procedure DrawItem(const DIRec: TDrawItemStruct);
    procedure CNMeasureItem(var Message: TWMMeasureItem); message CN_MEASUREITEM;
    procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;

    protected
    property Brush;
    property Default;
    property Action;
    procedure SetButtonStyle(ADefault: Boolean); overRide;
    procedure WMPaint(var Message: TMessage); message WM_PAINT;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;

    public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;

    published
    { Published declarations }
    property PicNormal: TBitMap read FNormalPic write setNormal;
    property PicHiLight: TBitMap read FHiLightPic write setHiLight;
    property PicPushed: TBitMap read FDownPic write setDown;
    property PicDisabled: TBitmap read FDisabledPic write setDisabled;
    property PicFocus: TBitmap read FFocusPic write setFocusPic;
    property FocusColor: TColor read FFocusColor write FFocusColor default clRed;
    property DoFocusText: Boolean read FDoText write FDoText default False;
    property DoFocusRect: Boolean read FDoRect write FDoRect default False;
    property Caption;
    property Enabled;
    property ParentShowHint;
    property ParentBiDiMode;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnEnter;
    property OnExit;

end;

procedure Register;

implementation

type
  TMyState = (bsUp, bsDown, bsHigh, bsDisabled);

procedure Register;
begin
  RegisterComponents('DelphiFr', [TPicButton]);
end;

constructor TPicButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle:= ControlStyle - [csSetCaption, csOpaque];
  FOver := False;
  FNormalPic  := TBitMap.Create;
  FHiLightPic := TBitmap.Create;
  FDownPic  := TBitmap.Create;
  FDisabledPic := TBitmap.Create;
  FFocusPic := TBitmap.Create;
  FDoText := False;
  FDoRect := False;
  FFocusColor := clRed;
  Constraints.MaxHeight := Height;
  Constraints.MinHeight := 0; // Height;
  Constraints.MaxWidth := Width;
  Constraints.MinWidth := Width;     // Klaus Constraints.MinHeight corrected by Constraints.MinWidth
end;

destructor TPicButton.Destroy;
begin
  inherited Destroy;
  FNormalPic.Free;
  FHiLightPic.Free;
  FDownPic.Free;
  FDisabledPic.Free;
  FFocusPic.Free;
end;

procedure TPicButton.Loaded;
begin
  inherited Loaded;
  if FNormalPic.Empty then Exit;
  Constraints.MaxHeight := FNormalPic.Height;
  Constraints.MinHeight := 0; // FNormalPic.Height;
  Constraints.MaxWidth := FNormalPic.Width;
  Constraints.MinWidth := FNormalPic.Width;    // Klaus  Constraints.MinHeight corrected by Constraints.MinWidth
  Width := FNormalPic.Width;
  Height := FNormalPic.Height;
end;

procedure TPicButton.SetButtonStyle(ADefault: Boolean);
begin
  if HandleAllocated then
    SendMessage(Handle, BM_SETSTYLE, BS_PUSHBUTTON or BS_OWNERDRAW, 1);
end;

procedure TPicButton.setNormal(Value: TBitMap);
begin
  FNormalPic.Assign(Value);
  if not FNormalPic.Empty then
    begin
      Constraints.MaxHeight := FNormalPic.Height;
      Constraints.MinHeight := 0; // FNormalPic.Height;
      Constraints.MaxWidth := FNormalPic.Width;
      Constraints.MinWidth := FNormalPic.Width;      // Klaus  Constraints.MinHeigt corrected by Constraints.MinWidth
      Height := FNormalPic.Height;
      Width := FNormalPic.Width;
    end;
  Invalidate;
end;

procedure TPicButton.setHiLight(Value: TBitMap);
begin
  FHiLightPic.Assign(Value);
end;

procedure TPicButton.setDown(Value: TBitMap);
begin
  FDownPic.Assign(Value);
end;

procedure TPicButton.setDisabled(Value: TBitMap);
begin
  FDisabledPic.Assign(Value);
end;

procedure TPicButton.setFocusPic(Value: TBitMap);
begin
  FFocusPic.Assign(Value);
end;
   
procedure TPicButton.CNMeasureItem(var Message: TWMMeasureItem);
begin
  Message.MeasureItemStruct^.itemWidth := Width;
  Message.MeasureItemStruct^.itemHeight := Height;
end;

procedure TPicButton.CNDrawItem(var Message: TWMDrawItem);
begin
  DrawItem(Message.DrawItemStruct^);
end;

procedure TPicButton.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
end;

procedure TPicButton.CMMouseLeave(var Message: TMessage);
begin
  FOver := False;
  if Enabled and Assigned(FHiLightPic) then
    Invalidate;
end;

procedure TPicButton.CMMouseEnter(var Message: TMessage);
begin
  FOver := True;
  if Enabled and Assigned(FHiLightPic) then
    Invalidate;
end;

procedure TPicButton.CMFontChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TPicButton.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TPicButton.WMPaint(var Message: TMessage);
var
  BmpDC: Integer;
  PaintS: TPaintStruct;
  cRect: TRect;
  PenO: Cardinal;
begin
  if not (csDesigning in ComponentState) then
    begin
      Inherited;
      Exit;
    end;

  Message.Result := 0;
  BeginPaint(Handle, PaintS);
  if FNormalPic.Empty then
    begin
      PenO := SelectObject(PaintS.hDC, CreatePen(PS_SOLID, 1, $999999));
      SetROP2(PaintS.hdc, R2_XORPEN);
      Rectangle(PaintS.hdc, 0, 0, Width, Height);
      DeleteObject(SelectObject(PaintS.hdc, PenO));
      EndPaint(Handle,PaintS);
      Exit;         // deactivated by Klaus
    end;

  BmpDC := CreateCompatibleDC(0);
  SelectObject(BmpDC, FNormalPic.Handle);
  BitBlt(PaintS.hdc, PaintS.rcPaint.Left, PaintS.rcPaint.Top,
           PaintS.rcPaint.Right, PaintS.rcPaint.Bottom, BmpDC,
           PaintS.rcPaint.Left, PaintS.rcPaint.Top, SRCCOPY);
  DeleteDC(BmpDC);

  if Length(Caption) > 0 then
    begin
      cRect := ClientRect;

      SelectObject(PaintS.hdc, Font.Handle);
      SetBkMode(PaintS.hdc, TRANSPARENT);
      DrawText(PaintS.hdc,PChar(Caption),-1,cRect,DT_SINGLELINE or DT_VCENTER or DT_CENTER);
    end;
  EndPaint(Handle,PaintS);
end;

procedure TPicButton.DrawItem(const DIRec: TDrawItemStruct);
var
  Pic: TBitmap;
  IsFocus: Boolean;
  State: TMyState;
  cRect: TRect;
begin
  if (csDesigning in ComponentState) then Exit;

  cRect := ClientRect;
  {I assign a State to get the correct Image}
  if not Enabled then State := bsDisabled else
  if DIRec.itemState and ODS_SELECTED <> 0 then
    State := bsDown else
  if FOver then
    State := bsHigh else
    State := bsUp;

  IsFocus := DIRec.itemState and ODS_FOCUS <> 0;

  Pic := FNormalPic;                     // added by Klaus
  SetTextColor(DIRec.hdc, Font.Color);   // added by Klaus
  case State of
  {set the Pic and cRect}
    bsUp: if IsFocus then
        if not FFocusPic.Empty then Pic := FFocusPic;
    bsDown: begin
          if not FDownPic.Empty then
            Pic := FDownPic else
            if not FHiLightPic.Empty then Pic := FHiLightPic;
          cRect := Rect(2,2,Width+2,Height+2)
          end;
    bsHigh: if not FHiLightPic.Empty then Pic := FHiLightPic;
    bsDisabled: begin
              if not FDisabledPic.Empty then Pic := FDisabledPic;
              SetTextColor(DIRec.hdc, clSilver);
              end;
  end;

  if not Pic.Empty then
    BitBlt(DIRec.hDC, 0, 0, Width, Height, Pic.Canvas.Handle, 0, 0, SRCCOPY);

  if IsFocus and FDoText then
    SetTextColor(DIRec.hdc, FFocusColor);

  SelectObject(DIRec.hDC, Font.Handle);
  SetBkMode(DiRec.hdc, TRANSPARENT);
  if Length(Caption) > 0 then
    DrawText(DIRec.hDC,PChar(Caption),-1,cRect,DT_SINGLELINE or DT_VCENTER or DT_CENTER);

  if IsFocus and FDoRect then
    begin
      InflateRect(cRect, -3, -3);
      windows.DrawFocusRect(DIRec.hdc, cRect);
    end;
end;

end.